@@ -1,4 +1,5 @@
-# 2022.3.20, usage:  import en |  spacy.snts("hello") 
+# 2022.5.28, redisgears's python is 3.7 , to be compatible 
+# 2022.3.20, usage:  import en |  spacy.nlp("hello")   
 import json,spacy,os,builtins
 from spacy.tokens import DocBin,Doc,Token
 from spacy.tokenizer import Tokenizer
@@ -54,25 +55,54 @@ def sqlitedoc(snt, table):
 	sqlitedoc.conn.commit()
 	return doc 
 
+def to_docbin(doc):
+	doc_bin= spacy.tokens.DocBin()
+	doc_bin.add(doc)
+	return doc_bin.to_bytes()
+
+def from_docbin(bs): 
+	return list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
+
 if not hasattr(spacy, 'nlp'):
-	from spacy.lang import en
-	spacy.sntbr		= (inst := en.English(), inst.add_pipe("sentencizer"))[0]
-	spacy.snts		= lambda essay, trim=True: [ snt.text.strip() if trim else snt.text for snt in  spacy.sntbr(essay).sents]
-	spacy.sntpid	= lambda essay: (pid:=0, [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid))[-1] for snt in  spacy.sntbr(essay).sents] )[-1]
-	spacy.sntpidoff	= lambda essay: (pid:=0, doc:=spacy.sntbr(essay), [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid, doc[snt.start].idx))[-1] for snt in  doc.sents] )[-1]
-	spacy.nlp		= spacy.load('en_core_web_sm')
-	spacy.frombs	= lambda bs: list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
-	spacy.tobs		= lambda doc: ( doc_bin:= spacy.tokens.DocBin(), doc_bin.add(doc), doc_bin.to_bytes())[-1]
-	spacy.tok		= lambda doc: ' '.join([t.text.strip() for t in doc])
-	spacy.nps		= lambda doc : {f"{doc[np.end-1].lemma_}/np:{np.text.lower()}" for np in doc.noun_chunks} #book/np:a book
-	spacy.trps		= lambda doc : {f"{t.dep_}_{t.head.pos_}_{t.pos_}:{t.head.lemma_} {t.lemma_}" for t in doc if t.pos_ not in ("PUNCT","PROPN","NUM","SPACE") and t.text.isalpha()} #'dobj_VERB_NOUN:open door';
-	spacy.getdoc	= lambda snt, getf=lambda snt: None : ( res := getf(snt), res if res else spacy.nlp(snt))[1] 
-	spacy.redisdoc	= lambda snt, r, prefix="": ( bs := r.get(prefix+snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), r.setnx(prefix+snt, spacy.tobs(doc)) if not bs else None )[1] # added 2022.3.29
+	spacy.nlp		= spacy.load('en_core_web_sm') 
 	spacy.merge_nps	= spacy.nlp.create_pipe("merge_noun_chunks")
 	spacy.nlp.tokenizer = custom_tokenizer(spacy.nlp)	#nlp.tokenizer.infix_finditer = infix_re.finditer
-	#print([t.text for t in nlp("It's 1.50, up-scaled haven't")])
-	# ['It', "'s", "'", '1.50', "'", ',', 'up-scaled', 'have', "n't"]
+	#print([t.text for t in nlp("It's 1.50, up-scaled haven't")]) # ['It', "'s", "'", '1.50', "'", ',', 'up-scaled', 'have', "n't"]
 
+def rgetdoc(rbs, snt, prefix="bs:", ttl:int=37200): 
+	''' bytes , 2022.5.28 '''
+	bs = rbs.get( prefix + snt) 
+	if bs: return Doc(spacy.nlp.vocab).from_bytes(bs)  # spacy.frombs(bs) 
+	doc = spacy.nlp(snt) 
+	rbs.setex(prefix + snt, ttl, doc.to_bytes()) 
+	return doc 
+
+def getdoc(snt, bs):  # execute("GET", f"bytes:{snt}") 
+	return Doc(spacy.nlp.vocab).from_bytes(bs) if bs else spacy.nlp(snt) 
+
+def sntbr(essay, trim:bool=False, with_pid:bool=False): 
+	''' added 2022.5.28 '''
+	from spacy.lang import en
+	if not hasattr(sntbr, 'inst'): 
+		sntbr.inst = en.English()
+		sntbr.inst.add_pipe("sentencizer")
+
+	doc = sntbr.inst(essay)
+	if not with_pid: return [ snt.text.strip() if trim else snt.text for snt in  doc.sents]
+
+	pid = 0 #spacy.sntpidoff	= lambda essay: (pid:=0, doc:=spacy.sntbr(essay), [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid, doc[snt.start].idx))[-1] for snt in  doc.sents] )[-1]
+	arr = []
+	for snt in  doc.sents:
+		if "\n" in snt.text: pid = pid + 1 
+		arr.append( (snt.text, pid) ) 
+	return arr 
+
+#token_split	= lambda sent: re.findall(r"[\w']+|[.,!?;]", sent) # return list
+def common_perc(snt="She has ready.", trans="She is ready."): 
+	toks = set([t.text for t in spacy.nlp.tokenizer(snt)])
+	return len([t for t in spacy.nlp.tokenizer(trans) if t.text in toks]) / (len(toks)+0.01)
+
+toks			= lambda doc:  [{'i':t.i, "head":t.head.i, 'lex':t.text, 'lem':t.lemma_, 'pos':t.pos_, 'tag':t.tag_, 'dep':t.dep_, "gpos":t.head.pos_, "glem":t.head.lemma_} for t in doc ] # JSONEachRow 
 pred_offset		= lambda doc:  (ar := [ t.i for t in doc if t.dep_ == "ROOT"], offset := ar[0] if len(ar) > 0 else 0,  offset/( len(doc) + 0.1) )[-1]
 postag			= lambda doc:  "_^ " + " ".join([ f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + " _$"
 non_root_verbs	= lambda doc:  [t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT'] 
@@ -101,7 +131,6 @@ hit_trp		= lambda t, _rel, _gov_dep:   _rel == trp_rel(t) and _gov_dep == (t.hea
 trp_high	= lambda doc, i, ihead :   "".join([ f"<b>{t.text_with_ws}</b>" if t.i in (i, ihead) else t.text_with_ws for t in doc ])
 lem_high	= lambda doc, lem :   "".join([ f"<b>{t.text_with_ws}</b>" if t.lemma_ == lem else t.text_with_ws for t in doc ]) # highlight the first lemma 
 
-
 def readline(infile, sepa=None):
 	with open(infile, 'r') as fp:
 		while True:
@@ -109,11 +138,9 @@ def readline(infile, sepa=None):
 			if not line: break
 			yield line.strip().split(sepa) if sepa else line.strip()
 
-[ setattr(builtins, k, v) for k, v in globals().items() if not k.startswith("_") and not '.' in k and not hasattr(builtins,k) ]
-#setattr(builtins, "spacy", spacy)
-			
+[setattr(builtins, k, v) for k, v in globals().items() if not k.startswith("_") and not '.' in k and not hasattr(builtins,k) ]
+#setattr(builtins, "spacy", spacy)			
 if __name__	== '__main__': 
-	print(spacy.sntpidoff("   English   is a internationaly language which becomes importantly for modern world.  In China, English is took to be."))
-
+	print(sntbr("   English   is a internationaly language which becomes importantly for modern world.  In China, English is took to be.", with_pid=True))
 #c:\users\zhang\appdata\local\programs\python\python38\lib\site-packages  
-#/home/ubuntu/.local/lib/python3.8/site-packages/en
+#/home/ubuntu/.local/lib/python3.8/site-packages/en
\ No newline at end of file
