@@ -1,4 +1,5 @@
-# 2022.2.13
+# 2022.5.11 updated , add sqlrow for EachRowJson
+# 2022.2.13 created 
 import requests,json, os, math,re,traceback
 from elasticsearch import Elasticsearch,helpers
 from collections import	Counter
@@ -143,6 +144,8 @@ config = {
 			"corpus": { "type": "keyword"},
 			"folder": { "type": "keyword"},
 			"head": { "type": "keyword"},
+			"attr": { "type": "keyword"},
+			"val": { "type": "keyword"}, # head, attr, val 
 			"chunk": { "type": "keyword"},
 			"type": { "type": "keyword"},
 			"fn": { "type": "keyword"},
@@ -187,12 +190,13 @@ config = {
 				 },
 
 			"err": {"type": "text",  "analyzer": "err_ana"},
-			"fd": { "type": "keyword"},
+			"fd": { "type": "keyword", "index": "false"},
 			"short_msg": { "type": "keyword", "index": "false"},
 			"arr": { "type": "keyword", "index": "false" }, # dim arr of dsk
 			"info": { "type": "keyword", "index": "false" ,"store": "false"},
 			"kw": { "type": "keyword", "index": "false" ,"store": "false", "ignore_above": 60},
 			"meta": { "type": "keyword", "index": "false" ,"store": "false"},
+			"dim": { "type": "keyword", "index": "false" },
 			"tc": {"type": "integer" , "index": "false"},
 			"sc": {"type": "integer" , "index": "false"},
 			"isum": {"type": "integer" , "index": "false"},
@@ -230,6 +234,12 @@ lemnum	= lambda w,cp: sql(f"select count(*) cnt from {cp} where lem = '{w}' and
 sqlsi	= lambda query: (si:=Counter(), [si.update({row[0]:1}) for row in sql(query)])[0]
 warmup  = lambda : requests.put(f"http://{eshost}:{esport}/_cluster/settings", json={"persistent": {"search.max_buckets": 1000000}}).text
 
+def sqlrow(query="select lem, count(*) cnt  from gzjc where type = 'tok' and pos != 'PUNCT' group by lem order by cnt desc limit 10"):
+	''' {'columns': [{'name': 'lem', 'type': 'keyword'}, {'name': 'cnt', 'type': 'long'}], 'rows': [['the', 7552], ['be', 5640], ['and', 3604], ['to', 3561], ['of', 3127], ['a', 2902], ['in', 2687], ['I', 2063], ['have', 1562], ['it', 1311]]} '''
+	res = requests.post(f"http://{eshost}:{esport}/_sql",json={"query": query}).json() 
+	columns = [ ar['name'] for ar in res['columns'] ]
+	return  [  dict(zip(columns, ar)) for ar in res['rows'] ] #[{'lem': 'the', 'cnt': 7552}, {'lem': 'be', 'cnt': 5640}, {'lem': 'and', 'cnt': 3604}, {'lem': 'to', 'cnt': 3561}, {'lem': 'of', 'cnt': 3127}, {'lem': 'a', 'cnt': 2902}, {'lem': 'in', 'cnt': 2687}, {'lem': 'I', 'cnt': 2063}, {'lem': 'have', 'cnt': 1562}, {'lem': 'it', 'cnt': 1311}]
+
 def readline(infile, sepa=None):
 	with open(infile, 'r') as fp:
 		while True:
@@ -352,6 +362,8 @@ def hybchunk(hyb:str='the _NNS of', index:str='gzjc', size:int= -1, topk:int=10)
 
 if __name__ == "__main__": 
 	print(es)
+	print ( sql("select lem, count(*) cnt  from gzjc where type = 'tok' and pos != 'PUNCT' group by lem order by cnt desc limit 10"))
+	print ( sqlrow("show tables")) 
 
 '''
 PUT twitter/_mapping 
